import java.io.*;

public class CteniDae {
  private FileReader fr;
  private BufferedReader in;
  public String jmenoSouboru;
  private boolean precteno = false;
  private boolean konecSouboru = false;

  private PipedOutputStream rouraVystup;
  private PipedInputStream rouraVstup;
  private DataOutputStream binRouraVystup;
  private DataInputStream binRouraVstup;

  CteniDae(String jmeno) {
    jmenoSouboru = new String(jmeno);
    try {
      // otevreni souboru
      fr = new FileReader(jmenoSouboru);
      in = new BufferedReader(fr);

      // otevreni roury - data pujdou binarne neformatovane
      rouraVystup = new PipedOutputStream();
      rouraVstup = new PipedInputStream(rouraVystup);
      binRouraVystup = new DataOutputStream(rouraVystup);
      binRouraVstup = new DataInputStream(rouraVstup);
    }
    catch (IOException e) {
      System.out.println("Chyba pri otvirani souboru nebo roury!");
    }
  }

  synchronized public void nacti() {
    while (precteno == true) {
      try {
        wait();
      }
      catch (InterruptedException e) { }
    }

    String radka;
    try {
      if ((radka = in.readLine()) != null) {
        int hodnota = Integer.valueOf(radka).intValue();
        System.out.print(jmenoSouboru + " precteno: " + hodnota + "  ");
        // zapsani do roury
        binRouraVystup.writeInt(hodnota);
      }
      else {
        konecSouboru = true;
      }
    }
    catch (IOException e) {
      System.out.println("Chyba pri cteni souboru nebo zapisu do roury!");
    }
    precteno = true;
    notifyAll();
  }

  synchronized public int predej() {
    while (precteno == false) {
      try {
        wait();
      }
      catch (InterruptedException e) { }
    }

    precteno = false;
    if (konecSouboru == false) {
      // cteni z roury
      try {
        int hodnota = binRouraVstup.readInt();
        System.out.println(jmenoSouboru + " predano: " + hodnota + "  ");
        notifyAll();
        return hodnota;
      }
      catch (IOException e) {
        System.out.println("Chyba pri cteni z roury!");
      }
    }
    else {
      Thread.currentThread().interrupt();
    }

    return 0;
  }

  protected void finalize() {
    try {
      fr.close();
      binRouraVstup.close();
      binRouraVystup.close();
    }
    catch (IOException e) {
      System.out.println("Chyba pri zavirani souboru!");
    }
  }
}